import argparse
import logging
import re
import sys

import boto

from postman import __version__

syslog_handler = logging.handlers.SysLogHandler(address='/dev/log')
syslog_handler.setFormatter(logging.Formatter('%(name)s %(levelname)s: %(message)s'))

log = logging.getLogger(__name__)
log.addHandler(syslog_handler)
log.setLevel(logging.INFO)

stdout_handler = logging.StreamHandler(sys.stdout)
stdout_handler.setFormatter(logging.Formatter('%(message)s'))

out = logging.getLogger(__name__)
out.addHandler(stdout_handler)
out.setLevel(logging.INFO)

log.debug("Logging started.")

legal_headers = [ "Accept-Language", "Bcc", "Cc", "Comments", "Content-Type",
    "Content-Transfer-Encoding", "Content-ID", "Content-Description",
    "Content-Disposition", "Content-Language", "Date", "DKIM-Signature",
    "DomainKey-Signature", "From", "In-Reply-To", "Keywords", "List-Archive",
    "List-Help", "List-Id", "List-Owner", "List-Post", "List-Subscribe",
    "List-Unsubscribe", "Message-Id", "MIME-Version", "Received", "References",
    "Reply-To", "Return-Path", "Sender", "Subject", "Thread-Index",
    "Thread-Topic", "To", "User-Agent" ]

def sanitize(args, msg):
    """ Transform any headers SES disallows in to X-Headers.

    Sanitize method adapted from this Perl snippet:
    http://www.evanhoffman.com/evan/2011/05/16/amazon-ses-illegal-header-errors/
    """
    cleanmsg = ""
    in_header = True

    # Avoids indented lines.
    hdr_re = re.compile("^([\w\d\-]+):")

    for line in msg.splitlines(True):
        # Two line breaks between header and message body.
        if re.match("^[\n\r]", line):
            # Do nothing once we've scanned the headers.
            in_header = False
        if in_header and hdr_re.match(line):
            if hdr_re.match(line).group(1) not in legal_headers:
                line = "X-%s" % line
        cleanmsg += line
    return cleanmsg

def cmd_send(args):
    ses = boto.connect_ses()
    log.info("Sending mail to: %s" % ", ".join(args.destinations))
    msg = sys.stdin.read()
    if args.sanitize:
        msg = sanitize(args, msg)
    log.debug("Message is: %s" % msg)
    try:
        r = ses.send_raw_email(msg, args.f, args.destinations)
        if r.get("SendRawEmailResponse", {}).get("SendRawEmailResult", {}).get("MessageId"):
            log.info("OK")
        else:
            log.error("ERROR: %s" % r)
    except boto.exception.BotoServerError as e:
        log.error("ERROR: %s" % e)

def cmd_verify(args):
    ses = boto.connect_ses()
    for email in args.email:
        ses.verify_email_address(email)
        log.info("Verification for %s sent." % email)


def cmd_list_verified(args):
    ses = boto.connect_ses()
    args.verbose = True

    addresses = ses.list_verified_email_addresses()
    addresses = addresses["ListVerifiedEmailAddressesResponse"]
    addresses = addresses["ListVerifiedEmailAddressesResult"]
    addresses = addresses["VerifiedEmailAddresses"]

    if not addresses:
        log.warn("No addresses are verified on this account.")
        return

    for address in addresses:
        log.info(address)


def cmd_show_quota(args):
    ses = boto.connect_ses()
    args.verbose= True

    data = ses.get_send_quota()["GetSendQuotaResponse"]["GetSendQuotaResult"]
    out.info("Max 24 Hour Send: %s" % data["Max24HourSend"])
    out.info("Sent Last 24 Hours: %s" % data["SentLast24Hours"])
    out.info("Max Send Rate: %s" % data["MaxSendRate"])


def cmd_show_stats(args):
    ses = boto.connect_ses()
    args.verbose = True

    data = ses.get_send_statistics()
    data = data["GetSendStatisticsResponse"]["GetSendStatisticsResult"]
    for datum in data["SendDataPoints"]:
        out.info("Complaints: %s" % datum["Complaints"])
        out.info("Timestamp: %s" % datum["Timestamp"])
        out.info("DeliveryAttempts: %s" % datum["DeliveryAttempts"])
        out.info("Bounces: %s" % datum["Bounces"])
        out.info("Rejects: %s" % datum["Rejects"])
        out.info("")


def cmd_delete_verified(args):
    ses = boto.connect_ses()
    for email in args.email:
        ses.delete_verified_email_address(email_address=email)
        out.info("Deleted %s" % email)


def main():
    parser = argparse.ArgumentParser(prog="postman", description="send an email via Amazon SES")
    parser.add_argument("--version", action="version", version="%%(prog)s %s" % __version__)
    parser.add_argument("--verbose", action="store_true")

    command_parsers = parser.add_subparsers(dest="command")

    # cmd: send
    parser_send = command_parsers.add_parser("send")
    parser_send.add_argument("-f",
        help="the address to send the message from, must be validated")
    parser_send.add_argument("--sanitize", action="store_true", 
        help="Sanitize headers. Convert illegal headers to X-Headers.")
    parser_send.add_argument("destinations", metavar="TO", type=str, nargs="+",
        help="a list of email addresses to deliver message to")

    # cmd: verify
    parser_send = command_parsers.add_parser("verify")
    parser_send.add_argument("email", nargs="+",
        help="an email address to verify for sending from")

    # cmd: list_verified
    command_parsers.add_parser("list_verified")

    # cmd: show_quota
    command_parsers.add_parser("show_quota")

    # cmd: show_stats
    command_parsers.add_parser("show_stats")

    # cmd: delete_verified
    parser_delete = command_parsers.add_parser("delete_verified")
    parser_delete.add_argument("email", nargs="+",
        help="verified email addresses that will be deleted from verification list")

    args = parser.parse_args()

    if args.verbose:
        log.setLevel(logging.DEBUG)
        out.setLevel(logging.DEBUG)

    {
        "send": cmd_send,
        "verify": cmd_verify,
        "list_verified": cmd_list_verified,
        "show_quota": cmd_show_quota,
        "show_stats": cmd_show_stats,
        "delete_verified": cmd_delete_verified
    }[args.command](args)
